@@ -1,7 +1,7 @@
 
-__version__ = "5.7"
+__version__ = "5.8"
 docs = r"https://crocodile.readthedocs.io/en/latest/"
 git = r"https://github.com/thisismygitrepo/crocodile"
 release_notes = """
-new methods to P
+Bug fixes
 """
