@@ -6,7 +6,7 @@ Crocodile Philosophy:
 Make Python even friendlier, by making available the common functionality for everyday use. Specifically,
 path management, file management, and other common tasks.
 A the risk of vandalizing the concept, to make Python more MATLAB-like, in that more libraries are loaded up at
-start time than basic arithmetic, but just enought to make it more useful for everyday errands.
+start time_produced than basic arithmetic, but just enought to make it more useful for everyday errands.
 Thus, the terseness of Crocodile makes Python REPL a proper shell
 
 The focus is on ease of use, not efficiency.
@@ -80,14 +80,15 @@ def validate_name(astring, replace='_'):
     return re.sub(r'^(?=\d)|\W', replace, str(astring))
 
 
-def install_n_import(package):
+def install_n_import(package, name=None):
     """imports a package and installs it first if not."""
     try:
         pkg = __import__(package)
         return pkg
     except ImportError:
         import subprocess
-        subprocess.check_call([sys.executable, "-m", "pip", "install", package])
+        if name is None: name = package
+        subprocess.check_call([sys.executable, "-m", "pip", "install", name])
     pkg = __import__(package)
     return pkg
 
@@ -224,7 +225,7 @@ class Save:
     @save_decorator(".pkl")
     def pickle(obj=None, path=None, r=False, **kwargs):
         """This is based on `dill` package. While very flexible, it comes at the cost of assuming so many packages are
-        loaded up and it happens implicitly. It often fails at load time and requires same packages to be reloaded first
+        loaded up and it happens implicitly. It often fails at load time_produced and requires same packages to be reloaded first
         . Compared to vanilla pickle, the former always raises an error when cannot pickle an object due to
         dependency. Dill however, stores all the required packages for any attribute object, but not the class itself,
         or the classes that it inherits (at least at with this version)."""
@@ -277,15 +278,15 @@ class Base(object):
             * Downs:
                 * It requires (in case of composed objects) that every sub-object is well-behaved and has the appropriate
                   state methods implemented.
-                * The libraries imported must be present at load time.
+                * The libraries imported must be present at load time_produced.
         * `itself` is False: means to save __getstate__.
             * Ups:
-                * very safe and unlikely to cause import errors at load time.
+                * very safe and unlikely to cause import errors at load time_produced.
                 * User is responsible for loading up the class.
             * Downs: the class itself is required later and the `from_pickled_state` method should be used to reload the instance again.
-            * __init__ method will be used again at reconstruction time of the object before the attributes are monkey-patched.
+            * __init__ method will be used again at reconstruction time_produced of the object before the attributes are monkey-patched.
             * It is very arduous to design __init__ method that is convenient (uses plethora of
-              default arguments) and works at the same time with no input at reconstruction time.
+              default arguments) and works at the same time_produced with no input at reconstruction time_produced.
 
         :param include_code: `save_code` will be called.
         :param r: recursive flag.
@@ -652,7 +653,8 @@ class List(Base, list):
         return res
 
     # ======================= Modify Methods ===============================
-    def flatten(self):
+    def flatten(self):  # AKA combine, concatenate
+        """The ouput is of unknown type. Items must support addition."""
         res = self.list[0]
         for item in self.list[1:]:
             res = res + item
@@ -700,7 +702,7 @@ class List(Base, list):
         :param verbose:
         :param desc:
         :param depth: apply the function to inner Lists
-        :param kwargs: a list of outputs each time the function is called on elements of the list.
+        :param kwargs: a list of outputs each time_produced the function is called on elements of the list.
         :return:
         """
         if depth > 1:
@@ -838,17 +840,12 @@ class Struct(Base, dict):
         :param dictionary: a dict, a Struct, None or an object with __dict__ attribute.
         """
         super(Struct, self).__init__()
-        if type(dictionary) is Struct:
-            dictionary = dictionary.dict
-        if dictionary is None:  # only kwargs were passed
-            final_dict = kwargs
+        if type(dictionary) is Struct: dictionary = dictionary.dict
+        if dictionary is None: final_dict = kwargs  # only kwargs were passed
         elif not kwargs:  # only dictionary was passed
-            if type(dictionary) is dict:
-                final_dict = dictionary
-            elif type(dictionary) == "mappingproxy":
-                final_dict = dict(dictionary)
-            else:
-                final_dict = dictionary.__dict__
+            if type(dictionary) is dict: final_dict = dictionary
+            elif dictionary.__class__.__name__ == "mappingproxy": final_dict = dict(dictionary)
+            else: final_dict = dictionary.__dict__
         else:  # both were passed
             final_dict = dictionary if type(dictionary) is dict else dictionary.__dict__
             final_dict.update(kwargs)
@@ -1011,6 +1008,21 @@ class Struct(Base, dict):
     def __iter__(self):  # used when list(~) is called or it is iterated over.
         return iter(self.dict.items())
 
+    def __delitem__(self, key):
+        del self.__dict__[key]
+
+    def delete(self, key=None, keys=None, criterion=None):
+        if key is not None:
+            del self.__dict__[key]
+        if keys is not None:
+            for key in keys:
+                del self.__dict__[key]
+        if criterion is not None:
+            for key in self.keys().list:
+                if criterion(self[key]):
+                    del self.__dict__[key]
+        return self
+
     def update(self, *args, **kwargs):
         """Accepts dicts and keyworded args
         """
@@ -1018,12 +1030,20 @@ class Struct(Base, dict):
         self.__dict__.update(new_struct.__dict__)
         return self
 
-    def apply(self, func):
-        func = self.evalstr(func)
+    def apply_to_keys(self, key_val_func):
+        res = dict()
         for key, val in self.items():
-            self[key] = func(val)
+            res[key_val_func(key, val)] = val
+        return Struct(res)
+
+    def apply_to_values(self, key_val_func):
+        for key, val in self.items():
+            self[key] = key_val_func(val)
         return self
 
+    def filter(self, key_val_func=None):
+        return Struct({key: self[key] for key, val in self.items() if key_val_func(key, val)})
+
     def inverse(self):
         return Struct({v: k for k, v in self.dict.items()})
 
