@@ -1,3 +1,4 @@
+import time
 
 from crocodile.core import Struct, np, os, sys, List, datetime, timestamp, randstr, validate_name, str2timedelta,\
     Save, Path, install_n_import, dill
@@ -114,6 +115,7 @@ def decrypt(token: bytes, key=None, pwd: str = None, salted=True) -> bytes:
 
 
 # =================================== File ============================================
+
 class Read(object):
     @staticmethod
     def read(path, **kwargs):
@@ -274,7 +276,7 @@ class P(type(Path()), Path):
         """
         path = self._resolve_path(folder=folder, name=name, path=path,
                                   default_name=self.absolute().name, rel2it=rel2it)
-        name, folder = self.name, self.parent
+        name, folder = path.name, path.parent
 
         if parents: folder.create(parents=True, exist_ok=True)
         slf = self.expanduser().resolve()
@@ -368,9 +370,11 @@ class P(type(Path()), Path):
         :param notfound: behaviour when file ``self`` to be read doesn't actually exist. Default: throw an error.
                 can be set to return `False` or any other value that will be returned if file not found.
         :param verbose:
+        :param readerror:
         :param kwargs:
         :return:
         """
+        _ = readerror
         if not self.exists():
             if notfound is FileNotFoundError:
                 raise FileNotFoundError(f"`{self}` is no where to be found!")
@@ -389,9 +393,10 @@ class P(type(Path()), Path):
             raise IOError
 
     def __call__(self, *args, **kwargs):
-        return self.start()
+        return self.start(*args, **kwargs)
 
-    def start(self):  # explore folders.
+    def start(self, opener=None):  # explore folders.
+        import subprocess
         if str(self).startswith("http") or str(self).startswith("www"):
             import webbrowser
             webbrowser.open(str(self))
@@ -400,24 +405,23 @@ class P(type(Path()), Path):
         # os.startfile(os.path.realpath(self))
         filename = self.expanduser().resolve().str
         if sys.platform == "win32":
-            os.startfile(filename)  # works for files and folders alike
+            if opener is None: tmp = f"powershell start '{filename}'"  # double quotes fail with cmd.
+            else: tmp = rf'powershell {opener} \'{self}\''
+            # os.startfile(filename)  # works for files and folders alike, but if opener is given, e.g. opener="start"
+            subprocess.Popen(tmp)  # fails for folders. Start must be passed, but is not defined.
         elif sys.platform == 'linux':
-            import subprocess
             opener = "xdg-open"
             subprocess.call([opener, filename])  # works for files and folders alike
         else:  # mac
             # os.system(f"open {filename}")
-            import subprocess
             subprocess.call(["open", filename])  # works for files and folders alike
         return self
 
-    def append_text(self, txt):
-        text = self.read_text()
-        text += txt
-        self.write_text(text)
+    def append_text(self, appendix):
+        self.write_text(self.read_text() + appendix)
         return self
 
-    def modify_text(self, txt, alt, newline=True):
+    def modify_text(self, txt, alt, newline=False):
         """
         :param txt: text to be searched for in the file. The line in which it is found will be up for change.
         :param alt: alternative text that will replace `txt`. Either a string or a function returning a string
@@ -439,7 +443,7 @@ class P(type(Path()), Path):
                     lines[idx] = alt if type(alt) is str else alt(line)
                 else:
                     lines[idx] = line.replace(txt, alt if type(alt) is str else alt(line))
-        if bingo is False:  lines.append(txt)  # txt not found, add it anyway.
+        if bingo is False:  lines.append(alt)  # txt not found, add it anyway.
         self.write_text("\n".join(lines))
         return self
 
@@ -458,8 +462,8 @@ class P(type(Path()), Path):
             return response.content
         # Alternative: from urllib import request; request.urlopen(url).read().decode('utf-8')
 
-    def read_refresh(self, refresh, expire="1w", save=Save.pickle, read=Read.read):
-        return Fridge(refresh=refresh, path=self, expire=expire, save=save, read=read)
+    def read_fresh_from(self, source_func, expire="1w", save=Save.pickle, read=Read.read):
+        return Fridge(source_func=source_func, path=self, expire=expire, save=save, read=read)
 
     # ================================ Path Object management ===========================================
     """ The default behaviour of methods that mutate the path object:
@@ -697,7 +701,7 @@ class P(type(Path()), Path):
         rep = "P:"
         if self.is_symlink():
             try: target = self.resolve()  # broken symolinks are funny, and almost always fail `resolve` method.
-            except: target = "BROKEN LINK " + str(self)
+            except Exception: target = "BROKEN LINK " + str(self)
             if target == self: target = str(target)  # avoid infinite recursions for broken links.
             rep += " Symlink '" + str(self) + "' ==> " + repr(target)
         elif self.is_absolute():
@@ -732,16 +736,16 @@ class P(type(Path()), Path):
 
     def time(self, which="m", **kwargs):
         """Meaning of ``which values``
-            * ``m`` time of modifying file ``content``, i.e. the time it was created.
-            * ``c`` time of changing file status (its inode is changed like permissions, path etc, but not content)
-            * ``a`` last time the file was accessed.
+            * ``m`` time_produced of modifying file ``content``, i.e. the time_produced it was created.
+            * ``c`` time_produced of changing file status (its inode is changed like permissions, path etc, but not content)
+            * ``a`` last time_produced the file was accessed.
 
-        :param which: Determines which time to be returned. Three options are availalable:
+        :param which: Determines which time_produced to be returned. Three options are availalable:
         :param kwargs:
         :return:
         """
-        time = {"m": self.stat().st_mtime, "a": self.stat().st_atime, "c": self.stat().st_ctime}[which]
-        return datetime.fromtimestamp(time, **kwargs)
+        timestamp_ = {"m": self.stat().st_mtime, "a": self.stat().st_atime, "c": self.stat().st_ctime}[which]
+        return datetime.fromtimestamp(timestamp_, **kwargs)
 
     def stats(self):
         """A variant of `stat` method that returns a structure with human-readable values."""
@@ -788,14 +792,14 @@ class P(type(Path()), Path):
         return self._return(P(str(self).replace('\\', '/').replace('//', '/')), inlieu)
 
     # ========================== override =======================================
-    def symlink_from(self, target=None, target_dir=None, verbose=False, overwrite=False):
+    def symlink_from(self, folder=None, file=None, verbose=False, overwrite=False):
         assert self.expanduser().exists(), "self must exist if this method is used."
-        if target is not None:
-            assert target_dir is None, "You can only pass source or source_dir, not both."
-            result = P(target)
+        if file is not None:
+            assert folder is None, "You can only pass source or source_dir, not both."
+            result = P(file).expanduser().absolute()
         else:
-            if target_dir is None: target_dir = P.cwd()
-            result = P(target_dir) / self.name
+            if folder is None: folder = P.cwd()
+            result = P(folder).expanduser().absolute() / self.name
         return result.symlink_to(self, verbose=verbose, overwrite=overwrite)
 
     def symlink_to(self, target=None, verbose=True, overwrite=False, orig=False):
@@ -813,9 +817,21 @@ class P(type(Path()), Path):
             if self.is_symlink() or self.exists():
                 # self.exists() is False for broken links even though they exist
                 self.delete(sure=True, verbose=verbose)
-        super(P, self.expanduser()).symlink_to(str(target))
+        import platform
+        from crocodile.meta import Terminal
+        if platform.system() == "Windows" and not Terminal.is_user_admin():  # you cannot create symlink without priviliages.
+            Terminal.run_code_as_admin(f" -c \"from pathlib import Path; Path(r'{self.expanduser()}').symlink_to(r'{str(target)}')\"")
+            time.sleep(0.5)  # give time_produced for asynch process to conclude before returning response.
+        else: super(P, self.expanduser()).symlink_to(str(target))
         if verbose: print(f"LINKED {repr(self)}")
         return P(target) if not orig else self
+    
+    def resolve(self, strict=False):
+        try:
+            res = super(P, self).resolve(strict=strict)
+        except OSError:
+            return self
+        return res
 
     def write_text(self, data: str, **kwargs):
         super(P, self).write_text(data, **kwargs)
@@ -825,6 +841,11 @@ class P(type(Path()), Path):
         super(P, self).write_bytes(data)
         return self
 
+    def touch(self, mode: int = 0o666, parents=True, exist_ok: bool = ...):
+        if parents: self.parent.create(parents=parents)
+        super(P, self).touch(mode=mode, exist_ok=exist_ok)
+        return self
+
     # ======================================== Folder management =======================================
     def create(self, parents=True, exist_ok=True, parent_only=False):
         """Creates directory while returning the same object
@@ -864,8 +885,7 @@ class P(type(Path()), Path):
             pass
 
         if not_in is not None:
-            for notin in not_in:
-                filters += [lambda x: str(notin) not in str(x)]
+            filters += [lambda x: all([str(notin) not in str(x) for notin in not_in])]
 
         # ============================ get generator of search results ========================================
 
@@ -1158,15 +1178,24 @@ class P(type(Path()), Path):
         pass
 
     def encrypt(self, key=None, pwd=None, folder=None, name=None, path=None, verbose=True,
-                append="_encrypted", inplace=False, orig=False):
+                append="_encrypted", inplace=False, orig=False, use_7z=False):
         """
         see: https://stackoverflow.com/questions/42568262/how-to-encrypt-text-with-a-password-in-python
         https://stackoverflow.com/questions/2490334/simple-way-to-encode-a-string-according-to-a-password
         """
         slf = self.expanduser().resolve()
         assert slf.is_file(), f"Cannot encrypt a directory. You might want to try `zip_n_encrypt`. {self}"
-        code = encrypt(msg=slf.read_bytes(), key=key, pwd=pwd)
         path = self._resolve_path(folder, name, path, slf.append(name=append).name)
+        if use_7z:
+            import crocodile.environment as env
+            path = path + '.7z'
+            if env.system == "Windows":
+                program = env.ProgramFiles.joinpath("7-Zip/7z.exe")
+                if not program.exists(): env.tm.run('winget install --name "7-zip" --Id "7zip.7zip" --source winget', shell="powershell")
+                env.tm.run(f"&'{program}' a '{path}' '{self}' -p{pwd}", shell="powershell")
+            else: raise NotImplementedError("7z not implemented for Linux")
+            return path
+        code = encrypt(msg=slf.read_bytes(), key=key, pwd=pwd)
         path.write_bytes(code)  # Fernet(key).encrypt(self.read_bytes()))
         if verbose: print(f"ENCRYPTED: {repr(slf)} ==> {repr(path)}.")
         if inplace: slf.delete(sure=True, verbose=verbose)
@@ -1340,26 +1369,31 @@ class MemoryDB:
 
 
 class Fridge:
-    """
-    This class helps to accelrate access to latest data coming from a rather expensive function,
+    """This class helps to accelrate access to latest data coming from a rather expensive function,
     Thus, if multiple methods from superior classses requested this within 0.1 seconds,
-    there will be no problem of API being a bottleneck reducing running time to few seconds
-    The class has two flavours, memory-based and disk-based variants.
-    """
-
-    def __init__(self, refresh, expire="1m", time=None, logger=None, path=None, save=Save.pickle, read=Read.read):
+    there will be no problem of API being a bottleneck reducing running time_produced to few seconds
+    The class has two flavours, memory-based and disk-based variants."""
+    def __init__(self, source_func, expire="1m", time_produced=None, logger=None, path=None, save=Save.pickle, read=Read.read):
         """
+        :param source_func: function that returns data
+        :param expire: time_produced after which the data is considered expired.
+        :param time_produced: creation time. If not provided, it will be taken from the source_func.
+        :param logger: logger to use.
+        :param path: path to save the data.
+        :param save: save method.
+        :param read: read method.
         """
         self.cache = None  # fridge content
-        self.time = time or datetime.now()  # init time
-        self.expire = expire  # how much time elapsed before
-        self.refresh = refresh  # function which when called returns a fresh object to be frozen.
+        self.time_produced = time_produced or datetime.now()  # init time_produced
+        self.expire = expire  # how much time_produced elapsed before
+        self.source_func = source_func  # function which when called returns a fresh object to be frozen.
         self.logger = logger
         self.path = P(path) if path else None  # if path is passed, it will function as disk-based flavour.
         self.save = save
         self.read = read
 
     def __getstate__(self):
+        """With this implementation, instances can be pickled and loaded up in different machine and still works."""
         state = self.__dict__.copy()
         if self.path is not None:
             state["path"] = self.path.rel2home()
@@ -1373,20 +1407,20 @@ class Fridge:
     @property
     def age(self):
         if self.path is None:
-            return datetime.now() - self.time
+            return datetime.now() - self.time_produced
         else:
             return datetime.now() - self.path.stats().content_mod_time
 
     def reset(self):
-        self.time = datetime.now()
+        self.time_produced = datetime.now()
 
     def __call__(self, fresh=False):
         """"""
         if self.path is None:  # Memory Fridge
             if self.cache is None or fresh is True or self.age > str2timedelta(self.expire):
-                self.cache = self.refresh()
-                self.time = datetime.now()
-                if self.logger: self.logger.debug(f"Updating / Saving data from {self.refresh}")
+                self.cache = self.source_func()
+                self.time_produced = datetime.now()
+                if self.logger: self.logger.debug(f"Updating / Saving data from {self.source_func}")
             else:
                 if self.logger: self.logger.debug(f"Using cached values. Lag = {self.age}.")
             return self.cache
@@ -1394,7 +1428,7 @@ class Fridge:
             if fresh or not self.path.exists() or self.age > str2timedelta(self.expire):
                 if self.logger: self.logger.debug(f"Updating & Saving {self.path} ...")
                 # print(datetime.now() - self.path.stats().content_mod_time, str2timedelta(self.expire))
-                self.cache = self.refresh()  # fresh order, never existed or exists but expired.
+                self.cache = self.source_func()  # fresh order, never existed or exists but expired.
                 self.save(obj=self.cache, path=self.path)
             elif self.age < str2timedelta(self.expire):
                 if self.cache is None:  # this implementation favours reading over pulling fresh at instantiation.
