@@ -1,8 +1,8 @@
-
 import logging
 import subprocess
 import time
-from crocodile.core import np, os, sys, timestamp, randstr, str2timedelta, datetime, Save,\
+# import types
+from crocodile.core import np, os, sys, timestamp, randstr, str2timedelta, datetime, Save, \
     dill, install_n_import
 from crocodile.file_management import P
 
@@ -464,6 +464,7 @@ def batcherv2(func_type='function', order=1):
             def __call__(self, *args, **kwargs):
                 output = [self.func(self, *items, *args[order:], **kwargs) for items in zip(*args[:order])]
                 return np.array(output)
+
         return Batch
 
 
@@ -499,10 +500,22 @@ class Terminal:
         def err(self):
             return self.output["stderr"]
 
+        @property
+        def success(self):
+            return self.output["returncode"] == 0
+
         @property
         def returncode(self):
             return self.output["returncode"]
 
+        @property
+        def as_path(self):
+            """More often than not, the output is a path."""
+            if self.err == "":
+                return P(self.op.rstrip())
+            else:
+                return None
+
         def capture(self):
             for key, val in self.std.items():
                 if val is not None and val.readable():
@@ -513,8 +526,9 @@ class Terminal:
             self.capture()
             print(f"Terminal Response:\nInput Command: {self.input}")
             for idx, (key, val) in enumerate(self.output.items()):
-                print(f"{idx} - {key} {'=' * 30}\n{val}")
-            print("-" * 50, "\n\n")
+                msg = f" {idx} - {key} "
+                print(f"{msg}".center(40, "-"), f"\n{val}")
+            print("=" * 50, "\n\n")
             return self
 
     def __init__(self, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE, elevated=False):
@@ -525,7 +539,7 @@ class Terminal:
         Shell
         Host
         * adding `start` to the begining of the command results in launching a new console that will not
-        inherit from the console python was launched from (e.g. conda enviroment), unlike when console path is ignored.
+        inherit from the console python was launched from (e.g. conda environment), unlike when console path is ignored.
 
         * `subprocess.Popen` (process open) is the most general command. Used here to create asynchronous job.
         * `subprocess.run` is a thin wrapper around Popen that makes it wait until it finishes the task.
@@ -635,7 +649,8 @@ class Terminal:
         my_list += cmds
         my_list = [item for item in my_list if item != ""]
         print("Meta.Terminal.run_async: Subprocess command: ", my_list)
-        w = subprocess.Popen(my_list, stdin=subprocess.PIPE, shell=True)  # stdout=self.stdout, stderr=self.stderr, stdin=self.stdin
+        w = subprocess.Popen(my_list, stdin=subprocess.PIPE,
+                             shell=True)  # stdout=self.stdout, stderr=self.stderr, stdin=self.stdin
         # returns Popen object, not so useful for communcation with an opened terminal
         return w
 
@@ -672,7 +687,7 @@ tb.sys.path.insert(0, r'{wdir}')
         # command = f'ipython {"-i" if interactive else ""} -c "{script}"'
 
     @staticmethod
-    def replicate_in_new_session(obj, execute=False):
+    def replicate_in_new_session(obj, execute=False, cmd=""):
         """Python brachnes off to a new window and run the function passed.
         context can be either a pickled session or the current file __file__"""
         # step 1: pickle the function
@@ -685,6 +700,7 @@ path = tb.P(r'{file}')
 obj = path.readit()
 path.delete(sure=True, verbose=False)
 obj{'()' if execute else ''}
+{cmd}
 """
         Terminal.run_script(script)
 
@@ -700,14 +716,88 @@ path.delete(sure=True, verbose=False)
 """
         Terminal().run_script(script=script)
 
+    @staticmethod
+    def is_user_admin():
+        """@return: True if the current user is an 'Admin' whatever that
+        means (root on Unix), otherwise False.
+        Warning: The inner function fails unless you have Windows XP SP2 or
+        higher. The failure causes a traceback to be printed and this
+        function to return False.
+        adopted from: https://stackoverflow.com/questions/19672352/how-to-run-script-with-elevated-privilege-on-windows
+        """
+        if os.name == 'nt':
+            import ctypes  # WARNING: requires Windows XP SP2 or higher!
+            try:
+                return ctypes.windll.shell32.IsUserAnAdmin()
+            except:
+                import traceback
+                traceback.print_exc()
+                print("Admin check failed, assuming not an admin.")
+                return False
+        else:  # Check for root on Posix
+            return os.getuid() == 0
+
+    @staticmethod
+    def run_code_as_admin(params):
+        _ = install_n_import("win32api", name="pypiwin32")
+        win32com = __import__("win32com", fromlist=["shell.shell.ShellExecuteEx"])
+        win32com.shell.shell.ShellExecuteEx(lpVerb='runas', lpFile=sys.executable, lpParameters=params)
+
+    @staticmethod
+    def run_as_admin(cmd_line=None, wait=True):
+        """Attempt to relaunch the current script as an admin using the same
+        command line parameters.  Pass cmdLine in to override and set a new
+        command.  It must be a list of [command, arg1, arg2...] format.
+        Set wait to False to avoid waiting for the sub-process to finish. You
+        will not be able to fetch the exit code of the process if wait is
+        False.
+        Returns the sub-process return code, unless wait is False in which
+        case it returns None.
+        @WARNING: this function only works on Windows.
+        adopted from: https://stackoverflow.com/questions/19672352/how-to-run-script-with-elevated-privilege-on-windows
+        """
+        if os.name != 'nt': raise RuntimeError("This function is only implemented on Windows.")
+        _ = install_n_import("win32api", name="pypiwin32")
+        import win32con
+        win32event = install_n_import("win32event")
+        win32process = install_n_import("win32process")
+        win32com = __import__("win32com", fromlist=["shell.shell.ShellExecuteEx"])
+        shell_execute_ex = win32com.shell.shell.ShellExecuteEx
+        win32com = __import__("win32com", fromlist=["shell.shellcon"])
+        shellcon = win32com.shell.shellcon
+        if cmd_line is None:
+            cmd_line = [sys.executable] + sys.argv
+        elif type(cmd_line) not in (tuple, list):
+            raise ValueError("cmdLine is not a sequence.")
+        cmd = '"%s"' % (cmd_line[0],)
+        # XXX TODO: isn't there a function or something we can call to massage command line params?
+        params = " ".join(['"%s"' % (x,) for x in cmd_line[1:]])
+        # print "Running", cmd, params
+        # ShellExecute() doesn't seem to allow us to fetch the PID or handle
+        # of the process, so we can't get anything useful from it. Therefore
+        # the more complex ShellExecuteEx() must be used.
+        # procHandle = win32api.ShellExecute(0, lpVerb, cmd, params, cmdDir, showCmd)
+        proce_info = shell_execute_ex(nShow=win32con.SW_SHOWNORMAL,
+                                      fMask=shellcon.SEE_MASK_NOCLOSEPROCESS,
+                                      lpVerb='runas',  # causes UAC elevation prompt.
+                                      lpFile=cmd,
+                                      lpParameters=params)
+        if wait:
+            proc_handle = proce_info['hProcess']
+            _ = win32event.WaitForSingleObject(proc_handle, win32event.INFINITE)
+            rc = win32process.GetExitCodeProcess(proc_handle)
+        else:
+            rc = None
+        return rc
+
 
 class SSH(object):
-    def __init__(self, username, hostname, ssh_key=None, pwd=None):
+    def __init__(self, username, hostname, sshkey=None, pwd=None):
         _ = False
         if _:
             super().__init__()
-        self.ssh_key = str(ssh_key) if ssh_key is not None else None
-        # no need to pass ssh_key if it was configured properly already
+        self.sshkey = str(sshkey) if sshkey is not None else None
+        # no need to pass sshkey if it was configured properly already
 
         import paramiko
         self.ssh = paramiko.SSHClient()
@@ -718,9 +808,10 @@ class SSH(object):
         self.ssh.connect(hostname=hostname,
                          username=username,
                          password=pwd,
-                         port=22, key_filename=self.ssh_key)
+                         port=22, key_filename=self.sshkey)
         self.sftp = self.ssh.open_sftp()
-        self.target_machine = "Windows" if self.run("$env:OS", printit=False).output["stdout"] == "Windows_NT" else "Linux"
+        self.target_machine = "Windows" if self.run("$env:OS", printit=False).output[
+                                               "stdout"] == "Windows_NT" else "Linux"
         # it must uses a python independent way to figure out the machine type to avoid circualrity below:
         import platform
         self.platform = platform
@@ -739,13 +830,13 @@ class SSH(object):
         """In SSH commands you need this:
         scp -r {self.get_key()} "{str(source.expanduser())}" "{self.username}@{self.hostname}:'{target}'"
         """
-        if self.ssh_key is not None:
-            return f"""-i "{str(P(self.ssh_key.expanduser()))}" """
+        if self.sshkey is not None:
+            return f"""-i "{str(P(self.sshkey).expanduser())}" """
         else:
             return ""
 
     @staticmethod
-    def copy_ssh_keys_to_remote(fqdn):
+    def copy_sshkeys_to_remote(fqdn):
         # Caveat, only windows is supported.
         # the following is a Windows Openssh alternative to ssh-copy-id
         Terminal().run(f'type $env:USERPROFILE\.ssh\id_rsa.pub | ssh {fqdn} "cat >> .ssh/authorized_keys"')
@@ -760,7 +851,7 @@ class SSH(object):
         return f"{os.getlogin()}@{self.platform.node()}"
 
     def open_console(self, new_window=True):
-        cmd = f"""ssh -i {self.ssh_key} {self.username}@{self.hostname}"""
+        cmd = f"""ssh -i {self.sshkey} {self.username}@{self.hostname}"""
         Terminal().run_async(cmd, new_window=new_window)
 
     def copy_env_var(self, name):
@@ -770,6 +861,7 @@ class SSH(object):
     def copy_from_here(self, source, target=None, zip_n_encrypt=False):
         pwd = None
         if zip_n_encrypt:
+            print(f"ZIPPING & ENCRYPTING".center(80, "="))
             pwd = randstr(length=10, safe=True)
             source = P(source).expanduser().zip_n_encrypt(pwd=pwd)
 
@@ -777,13 +869,15 @@ class SSH(object):
             target = P(source).collapseuser()
             assert target.is_relative_to("~"), f"If target is not specified, source must be relative to home."
             target = target.as_posix()
-        print(f"Step 1: Creating Target directory {target} @ remote machine.")
 
+        print("\n" * 3, f"Creating Target directory {target} @ remote machine.".center(80, "="))
         resp = self.runpy(f'print(tb.P(r"{target}").expanduser().parent.create())')
+
         remotepath = P(resp.op or "").joinpath(P(target).name).as_posix()
-        print(f"SENT `{source}` ==> `{remotepath}`")
+        print(f"SENT `{source}` ==> `{remotepath}`".center(80, "="))
         self.sftp.put(localpath=P(source).expanduser(), remotepath=remotepath)
         if zip_n_encrypt:
+            print(f"UNZIPPING & DECRYPTING".center(80, "="))
             resp = self.runpy(f"""tb.P(r"{remotepath}").expanduser().decrypt_n_unzip(pwd="{pwd}", inplace=True)""")
             source.delete(sure=True)
             return resp
@@ -813,13 +907,13 @@ class Log(object):
      is printing this message, in addition to many other concerns.
 
      Advantages of using instances of this class: You do not need to worry about object pickling process by modifing
-     the __getstate__ method of the class that will own the return_str. This is the case because loggers lose access
-     to the file return_str when unpickled, so it is better to instantiate them again.
+     the __getstate__ method of the class that will own the logger. This is the case because loggers lose access
+     to the file logger when unpickled, so it is better to instantiate them again.
      Logger can be pickled, but its handlers are lost, so what's the point? no perfect reconstruction.
      Additionally, this class keeps track of log files used, append to them if they still exist.
 
      Implementation detail: the design favours composition over inheritence. To counter the inconvenience
-      of having extra typing to reach the return_str, a property `return_str` was added to Base class to refer to it."""
+      of having extra typing to reach the logger, a property `logger` was added to Base class to refer to it."""
 
     def __init__(self, dialect=["colorlog", "logging", "coloredlogs"][0],
                  name=None, file: bool = False, file_path=None, stream=True, fmt=None, sep=" | ",
@@ -856,8 +950,8 @@ class Log(object):
     def critical(self, msg):
         return self.logger.critical(msg)
 
-    def set_level(self, level, which=["return_str", "stream", "file", "all"][0]):
-        if which in {"return_str", "all"}: self.logger.setLevel(level)
+    def set_level(self, level, which=["logger", "stream", "file", "all"][0]):
+        if which in {"logger", "all"}: self.logger.setLevel(level)
         if which in {"stream", "all"}: self.get_shandler().setLevel(level)
         if which in {"file", "all"}: self.get_fhandler().setLevel(level)
 
@@ -865,23 +959,27 @@ class Log(object):
         shandlers = []
         for handler in self.logger.handlers:
             if "StreamHandler" in str(handler):
-                if first: return handler
-                else: shandlers.append(handler)
+                if first:
+                    return handler
+                else:
+                    shandlers.append(handler)
         return shandlers
 
     def get_fhandler(self, first=True):
         fhandlers = []
         for handler in self.logger.handlers:
             if "FileHandler" in str(handler):
-                if first: return handler
-                else: fhandlers.append(handler)
+                if first:
+                    return handler
+                else:
+                    fhandlers.append(handler)
         return fhandlers
 
     @property
     def file(self):
         return P(self.specs["file_path"]) if self.specs["file_path"] else None
 
-    def _install(self):  # populates self.return_str attribute according to specs and dielect.
+    def _install(self):  # populates self.logger attribute according to specs and dielect.
         if self.specs["file"] is False and self.specs["stream"] is False:
             self.logger = Null()
         elif self.dialect == "colorlog":
@@ -901,11 +999,11 @@ class Log(object):
         self._install()
 
     def __getstate__(self):
-        # return_str can be pickled without this method,
+        # logger can be pickled without this method,
         # but its handlers are lost, so what's the point? no perfect reconstruction.
         state = self.__dict__.copy()
         state["specs"] = state["specs"].copy()
-        del state["return_str"]
+        del state["logger"]
         if self.specs["file_path"] is not None:
             state["specs"]["file_path"] = P(self.specs["file_path"]).expanduser()
         return state
@@ -988,7 +1086,7 @@ class Log(object):
     @staticmethod
     def get_logger(name=None, file=False, file_path=None, stream=True, fmt=None, sep=" | ",
                    s_level=logging.DEBUG, f_level=logging.DEBUG, l_level=logging.DEBUG):
-        """Basic Python return_str."""
+        """Basic Python logger."""
         logger = Log.get_base_logger(logging, name, l_level)
         fmt = logging.Formatter(fmt or Log.get_format(sep))
         Log.add_handlers(logger, logging, file, f_level, file_path, fmt, stream, s_level)
@@ -1006,9 +1104,9 @@ class Log(object):
 
     @staticmethod
     def add_handlers(logger, module, file, f_level, file_path, fmt, stream, s_level):
-        if file or file_path:  # ==> create file handler for the return_str.
+        if file or file_path:  # ==> create file handler for the logger.
             Log.add_filehandler(logger, file_path=file_path, fmt=fmt, f_level=f_level)
-        if stream:  # ==> create stream handler for the return_str.
+        if stream:  # ==> create stream handler for the logger.
             Log.add_streamhandler(logger, s_level, fmt, module=module)
 
     @staticmethod
@@ -1023,7 +1121,7 @@ class Log(object):
     @staticmethod
     def add_filehandler(logger, file_path=None, fmt=None, f_level=logging.DEBUG, mode="a", name="myFileHandler"):
         if file_path is None:
-            file_path = P.tmpfile(name="return_str", suffix=".log", folder="loggers")
+            file_path = P.tmpfile(name="logger", suffix=".log", folder="loggers")
         fhandler = logging.FileHandler(filename=str(file_path), mode=mode)
         fhandler.setFormatter(fmt=fmt)
         fhandler.setLevel(level=f_level)
@@ -1063,7 +1161,7 @@ def accelerate(func, ip):
     * func must be defined outside that context.
 
     To accelerate IO-bound process, use multithreading. An example of that is somthing very cheap to process,
-    but takes a long time to be obtained like a request from server. For this, multithreading launches all threads
+    but takes a long time_produced to be obtained like a request from server. For this, multithreading launches all threads
     together, then process them in an interleaved fashion as they arrive, all will line-up for same processor,
     if it happens that they arrived quickly.
 
@@ -1097,8 +1195,8 @@ class Scheduler:
         """
         :param wait: repeat the cycle every this many minutes.
         """
-        self.routine = routine  # main routine to be repeated every `wait` time.
-        self.occasional = occasional  # routine to be repeated every `other` time.
+        self.routine = routine  # main routine to be repeated every `wait` time_produced.
+        self.occasional = occasional  # routine to be repeated every `other` time_produced.
         self.exception_handler = exception if exception is not None else lambda ex: None
         self.wind_down = wind_down
         # routine to be run when an error occurs, e.g. save object.
@@ -1107,7 +1205,7 @@ class Scheduler:
         self.cycles = runs  # how many times to run the routine. defaults to infinite.
         self.logger = logger or Log(name="SchedulerAutoLogger" + randstr())
         self.history = []
-        self._start_time = None  # begining of a session (local time)
+        self._start_time = None  # begining of a session (local time_produced)
         self.total_count = 0
         self.count = 0
 
@@ -1117,11 +1215,11 @@ class Scheduler:
         self._start_time = datetime.now()
         wait_time = str2timedelta(self.wait).total_seconds()
         import pandas as pd
-        until = pd.to_datetime(until)  # (local time)
+        until = pd.to_datetime(until)  # (local time_produced)
 
         while datetime.now() < until and self.count < self.cycles:
             # 1- Opening Message ==============================================================
-            time1 = datetime.now()  # time before calcs started.  # use  fstring format {x:<10}
+            time1 = datetime.now()  # time_produced before calcs started.  # use  fstring format {x:<10}
             msg = f"Starting Cycle  {self.count: 4d}. Total Run Time = {str(datetime.now() - self._start_time)}."
             self.logger.info(msg + f" UTC Time: {datetime.utcnow().isoformat(timespec='minutes', sep=' ')}")
 
@@ -1140,7 +1238,7 @@ class Scheduler:
 
             # 4- Conclude Message ============================================================
             self.count += 1
-            time_left = int(wait_time - (datetime.now() - time1).total_seconds())  # take away processing time.
+            time_left = int(wait_time - (datetime.now() - time1).total_seconds())  # take away processing time_produced.
             time_left = time_left if time_left > 0 else 1
             self.logger.info(f"Finishing Cycle {self.count - 1: 4d}. "
                              f"Sleeping for {self.wait} ({time_left} seconds left)\n" + "-" * 50)
@@ -1155,19 +1253,19 @@ class Scheduler:
             if self.count >= self.cycles:
                 stop_reason = f"Reached maximum number of cycles ({self.cycles})"
             else:
-                stop_reason = f"Reached due stop time ({until})"
+                stop_reason = f"Reached due stop time_produced ({until})"
             self.record_session_end(reason=stop_reason)
 
     def record_session_end(self, reason="Unknown"):
-        """It is vital to record operation time to retrospectively inspect market status at session time."""
+        """It is vital to record operation time_produced to retrospectively inspect market status at session time_produced."""
         self.total_count += self.count
         end_time = datetime.now()  # end of a session.
         time_run = end_time - self._start_time
         self.history.append([self._start_time, end_time, time_run, self.count])
         self.logger.critical(f"\nScheduler has finished running a session. \n"
-                             f"start  time: {str(self._start_time)}\n"
-                             f"finish time: {str(end_time)} .\n"
-                             f"time    ran: {str(time_run)} | wait time {self.wait}  \n"
+                             f"start  time_produced: {str(self._start_time)}\n"
+                             f"finish time_produced: {str(end_time)} .\n"
+                             f"time_produced    ran: {str(time_run)} | wait time_produced {self.wait}  \n"
                              f"cycles  ran: {self.count}  |  Lifetime cycles: {self.total_count} \n"
                              f"termination: {reason} \n" + "-" * 100)
 
