@@ -15,14 +15,15 @@ from crocodile import file_management as _fm
 from crocodile import meta
 from crocodile.meta import logging, subprocess, time
 
-from crocodile import plot_management as _pm
-from crocodile.plot_management import plt, enum, FigureManager
+from crocodile import matplotlib_management as _pm
+from crocodile.matplotlib_management import plt, enum, FigureManager
 
 import pandas as pd
 _ = pd
 
-Base, timestamp, randstr = core.Base, core.timestamp, core.randstr
-Struct, DisplayData, str2timedelta, Save, List = core.Struct, core.DisplayData, core.str2timedelta, core.Save, core.List
+Base, timestamp, randstr, validate_name = core.Base, core.timestamp, core.randstr, core.validate_name
+Struct, Display, str2timedelta, Save, List = core.Struct, core.Display, core.str2timedelta, core.Save, core.List
+install_n_import = core.install_n_import
 
 P, Read, Compression = _fm.P, _fm.Read, _fm.Compression
 Fridge, MemoryDB = _fm.Fridge, _fm.MemoryDB
@@ -40,7 +41,7 @@ L = List
 tmp = P.tmp
 E = Experimental
 
-_ = Base, timestamp, Save, Terminal, List, Struct, DisplayData
+_ = Base, timestamp, Save, Terminal, List, Struct, Display
 _ = datetime, dt, os, sys, np, copy, random, dill
 
 _ = P, Read, Compression, string, sys  # , Path
